let { createElement, Suspense, useState, useEffect } = require('react');
let PropTypes = require('prop-types');

let subscriptions = new Set();

function Route(props) {

    let [visible, setVisible] = useState(window.location.pathname === props.href);

    useEffect(() => {
        let handleState = () => setVisible(window.location.pathname === props.href);
        window.addEventListener('popstate', handleState);
        subscriptions.add(handleState);
        return () => {
            window.removeEventListener('popstate', handleState);
            subscriptions.delete(handleState);
        };
    }, [props.href]);

    let Component = props.fallback ? createElement(Suspense, { fallback: props.fallback }, props.children) : props.children;
    return visible ? Component : null;
}

Route.propTypes = {
    href: PropTypes.string.isRequired,
    fallback: PropTypes.element,
}

module.exports.Route = Route;

function Link(props) {
    let handleClick = (event) => {
        if(event.ctrlKey)
            return;
        event.preventDefault();
        window.history.pushState(null, null, props.href);
        subscriptions.forEach(subscription => subscription());
    };
    return createElement('a', { href: props.href, onClick: handleClick }, props.children);
}

Link.propTypes = {
    href: PropTypes.string.isRequired,
}

module.exports.Link = Link;

module.exports.useHistory = function useHistory() {
    let push = (href) => {
        window.history.pushState(null, null, href);
        subscriptions.forEach(subscription => subscription());
    };
    let back = () => window.history.back();

    return {
        push: push,
        back: back,
    };
}
